/*
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "config.h"
#include "plasmax2godialog.h"
#include <QGraphicsWidget>
#include <QGraphicsLinearLayout>
#include <QDebug>
#include "plasmoidhovereditem.h"
#include "fsitem.h"
#include <plasma/widgets/scrollwidget.h>

#ifdef HAVE_PLASMA_SEPARATOR
#include <plasma/widgets/separator.h>
#endif

#include <plasma/widgets/iconwidget.h>
#include <plasma/widgets/label.h>
#include <QGraphicsSceneMouseEvent>
#include <QProcess>
#include <QTimer>
#include <KUser>
#include "x2goplasmoid.h"

PlasmaX2goDialog::PlasmaX2goDialog(QObject* parent): QObject(parent),m_widget(0)
{
	m_applet=(x2goplasmoid*)parent;
	initDialog();
}

PlasmaX2goDialog::~PlasmaX2goDialog()
{
}

void PlasmaX2goDialog::initDialog()
{
	m_widget=new QGraphicsWidget(m_applet);
	m_widget->installEventFilter(this);
	m_widget->setMinimumSize(150,180);
	m_widget->setPreferredSize(250,300);

	QGraphicsLinearLayout* lay=new QGraphicsLinearLayout(Qt::Vertical, m_widget);
	QGraphicsLinearLayout* headLay=new QGraphicsLinearLayout(Qt::Horizontal);
	Plasma::IconWidget* iconWidg=new Plasma::IconWidget(m_widget);
	iconWidg->setMaximumHeight(18);
	iconWidg->setIcon("x2go");
	Plasma::Label* headLabel=new Plasma::Label(m_widget);
	headLabel->setText(i18n("X2Go Plasmoid"));

	headLay->addItem(iconWidg);
	headLay->addItem(headLabel);
	headLay->addStretch();

	lay->addItem(headLay);
#ifdef HAVE_PLASMA_SEPARATOR
	lay->addItem((new Plasma::Separator(m_widget)));
#endif
	Plasma::ScrollWidget* widg=new Plasma::ScrollWidget(m_widget);
	widg->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
	itemWidget=new QGraphicsWidget(0);
	widg->setWidget(itemWidget);
	itemLayout=new QGraphicsLinearLayout(Qt::Vertical,itemWidget);
	//itemLayout->addStretch(99);
	lay->addItem(widg);
	lay->addStretch(99);

	PlasmoidHoveredItem* suspItem=new PlasmoidHoveredItem(m_widget);
	suspItem->setText(i18n("Suspend session"));
	suspItem->setIcon("system-lock-screen");
	suspItem->installEventFilter(this);
	suspItem->setMaximumHeight(32);
	//suspItem->setToolTip(i18n("Click to suspend session"));
	lay->addItem(suspItem);
	QString sessionId=getenv("X2GO_SESSION");

	if(sessionId.length()<=0)
	{
		suspItem->setEnabled(false);
		return;
	}
	connect(suspItem,SIGNAL(clicked()),this,SLOT(slotSuspendSession()));
	//slotUpdateItems();
	QTimer::singleShot(2000, this, SLOT(slotUpdateItems()));
}

bool PlasmaX2goDialog::eventFilter(QObject* obj, QEvent* event)
{
	PlasmoidHoveredItem* item = qobject_cast<PlasmoidHoveredItem*>(obj);
	if (item) {
		switch (event->type())
		{
			case QEvent::GraphicsSceneHoverLeave:
			{
				item->hoverLeave();
			}
			break;
			case QEvent::GraphicsSceneHoverEnter:
			{
				item->hoverEnter();
			}
			break;
			case QEvent::GraphicsSceneMousePress:
			{
				QGraphicsSceneMouseEvent *ev=static_cast<QGraphicsSceneMouseEvent *>(event);
				if (ev->button() == Qt::LeftButton)
				{
					item->buttonPressed();
					return true;
				}
			}
			default:
				break;
		}
	}
	return false;
}

void PlasmaX2goDialog::slotSuspendSession()
{
	QProcess::execute("x2gosuspend-session "+m_applet->getSessionId());
	m_applet->hidePopup();
}

void PlasmaX2goDialog::slotUpdateItems()
{
	updateItemProc=new QProcess(this);
	connect(updateItemProc,SIGNAL(finished(int,QProcess::ExitStatus)),this,
	        SLOT(slotUpdateItemsProcFinished(int,QProcess::ExitStatus)));
	updateItemProc->start("x2golistmounts " +m_applet->getSessionId());
}

void PlasmaX2goDialog::slotUpdateItemsProcFinished(int, QProcess::ExitStatus exitStatus)
{
	if (exitStatus==QProcess::NormalExit)
	{
		QString output=updateItemProc->readAllStandardOutput();
		QStringList lines=output.split("\n");
		QStringList actItems;
		KUser user;
		foreach(QString line,lines)
		{
			QStringList lstArr=line.split("|");
			if (lstArr.count()>=2)
			{
				lstArr[1].replace(" ","");
				if ((lstArr[1].indexOf("/tmp/spool_"+user.loginName())!=0) &&
				    (lstArr[1].indexOf("/tmp/mime_"+user.loginName())!=0) &&
				    (lstArr[1].indexOf("/tmp/.x2go-"+user.loginName()+"/spool")!=0) &&
				    (lstArr[1].indexOf("/tmp/.x2go-"+user.loginName()+"/mimebox")!=0)
				   )
					actItems<<lstArr[1];
			}
		}
		updateMountedItems(actItems);
	}
	QTimer::singleShot(2000, this, SLOT(slotUpdateItems()));
}

void PlasmaX2goDialog::updateMountedItems(QStringList newItems)
{
	foreach (QString item, mountedItems())
	{
		if (!newItems.contains(item))
			removeItem(item);
	}
	foreach (QString item, newItems)
	{
		if (!mountedItems().contains(item))
			appendItem(item);
	}
}

void PlasmaX2goDialog::appendItem(QString itemText)
{
	FsItem* item=new FsItem(itemWidget);
	itemLayout->insertItem(0,item);
	item->installEventFilter(this);
	item->setText(itemText);

	connect(item,SIGNAL(itemClicked(QString)),this,SLOT(slotOpenFolder(QString)));
	connect(item,SIGNAL(itemEject(QString)),this,SLOT(slotEjectFolder(QString)));

	// choose icon
	KUser user;
	if ((itemText.indexOf("/tmp/"+user.loginName()+"_media/cd/")==0) || (itemText.indexOf("/tmp/.x2go-"+user.loginName()+"/media/cd/")==0))
	{
		item->setIcon("media-optical");
	}
	else if ((itemText.indexOf("/tmp/"+user.loginName()+"_media/rm/")==0) || (itemText.indexOf("/tmp/.x2go-"+user.loginName()+"/media/rm/")==0))
	{
		item->setIcon("drive-removable-media-usb-pendrive");
	}
	else
		item->setIcon("drive-harddisk");
	m_applet->update();
	m_widget->update();
	m_applet->showPopup();
}

void PlasmaX2goDialog::removeItem(QString itemText)
{
	int itemCnt=itemLayout->count();
	for (int i=0;i<itemCnt;++i)
	{
		FsItem* item = (FsItem*)(itemLayout->itemAt(i));
		if (item->text()==itemText)
		{
			item->close();
			delete item;
			break;
		}
	}
	m_applet->update();
	m_widget->update();
}

QStringList PlasmaX2goDialog::mountedItems()
{
	QStringList items;
	int itemCnt=itemLayout->count();
	for (int i=0;i<itemCnt;++i)
	{
		FsItem* item = (FsItem*)(itemLayout->itemAt(i));
		items<<item->text();
	}
	return items;
}

void PlasmaX2goDialog::slotOpenFolder(const QString& folder)
{
	QProcess::startDetached("dolphin "+folder);
	m_applet->hidePopup();
}

void PlasmaX2goDialog::slotEjectFolder(const QString& folder)
{
	QStringList parts=folder.split("/", QString::SkipEmptyParts);
	if (parts.count()<=0)
		return;
	QProcess::startDetached("x2goumount-session "+m_applet->getSessionId()+" "+parts.last());
	m_applet->hidePopup();
}

